package com.chinanetcenter.wcs.android.slice;

import android.util.Log;

import com.chinanetcenter.wcs.android.utils.WCSLogUtil;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Locale;

public class Block {

	private static final long MAX_BLOCK_SIZE = 4 * 1024 * 1024;
	private static long sDefaultBlockSize = MAX_BLOCK_SIZE;
	private static final long MIN_BLOCK_SIZE = 1 * 1024 * 1024;
	public static final int SLICE_SIZE = 256 * 1024;

	private RandomAccessFile mRandomAccessFile;
	private long mStart;
	private long mSize;
	private int mSliceIndex;
	private long mOriginalFileSize;
	private String mFileName;

	Block(RandomAccessFile randomAccessFile, String fileName, long start, long blockSize) throws IOException {
		mRandomAccessFile = randomAccessFile;
		mOriginalFileSize = randomAccessFile.length();
		mFileName = fileName;
		mStart = start;
		mSize = blockSize;
	}

	public static Block[] blocks(File file) {
		return blocks(file, MAX_BLOCK_SIZE);
	}

	public static Block[] blocks(File file, long defaultBlockSize) {
		RandomAccessFile randomAccessFile = null;
		try {
			randomAccessFile = new RandomAccessFile(file, "r");
		} catch (FileNotFoundException e) {
			Log.e("CNCLog", "file not found : " + file);
			return null;
		}

		long fileSize = 0;
		try {
			fileSize = randomAccessFile.length();
		} catch (IOException e) {
			e.printStackTrace();
		}
		if (fileSize == 0) {
			return null;
		}

		if (defaultBlockSize < MIN_BLOCK_SIZE || defaultBlockSize > MAX_BLOCK_SIZE) {
			sDefaultBlockSize = MAX_BLOCK_SIZE;
		} else {
			sDefaultBlockSize = defaultBlockSize;
		}

		int blockCount = (int) ((fileSize + sDefaultBlockSize - 1) / sDefaultBlockSize);
		WCSLogUtil.d(String.format(Locale.CHINA, "file size : %s, block count : %s", fileSize, blockCount));
		Block[] blocks = new Block[blockCount];
		for (int i = 0; i < blockCount; i++) {
			long blockSize = sDefaultBlockSize;
			if (i + 1 == blockCount) {
				long remain = fileSize % sDefaultBlockSize;
				blockSize = remain == 0 ? sDefaultBlockSize : remain;
			}
			try {
				blocks[i] = new Block(randomAccessFile, file.getName(), i * sDefaultBlockSize, blockSize);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return blocks;
	}

	public Slice moveToNext() {
		return getSlice(mSliceIndex++);
	}

	public void setIndex(int index) {
		mSliceIndex = index;
	}

	public int getIndex() {
		return mSliceIndex;
	}
	
	public Slice moveToIndex(int index) {
		mSliceIndex = index;
		return moveToNext();
	}

	public Slice lastSlice() {
		return getSlice(mSliceIndex - 1);
	}

	private Slice getSlice(int index) {
		long offset = mStart + index * SLICE_SIZE;
		if (index * SLICE_SIZE >= mSize) {
			return null;
		}
		int sliceSize = SLICE_SIZE;
		if ((offset + SLICE_SIZE) > (mStart + mSize)) {
			sliceSize = (int) (mSize % SLICE_SIZE);
		}
		byte[] sliceData = new byte[sliceSize];
		try {
			mRandomAccessFile.seek(offset);
			mRandomAccessFile.read(sliceData, 0, sliceSize);
			WCSLogUtil.d("offset : " + offset + "; slice size : " + sliceSize);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return new Slice(index * SLICE_SIZE, sliceData);
	}

	public long size() {
		return mSize;
	}

	public String toString() {
		JSONObject jsonObject = new JSONObject();
		try {
			jsonObject.put("start", mStart);
			jsonObject.put("size", mSize);
			jsonObject.put("slice index", mSliceIndex);
		} catch (JSONException e) {
			e.printStackTrace();
		}
		try {
			return jsonObject.toString(4);
		} catch (JSONException e) {
			e.printStackTrace();
		}
		return "Block<>";
	}

	public long getOriginalFileSize() {
		return mOriginalFileSize;
	}

	public String getOriginalFileName() {
		return mFileName;
	}

}
